@@ -5,6 +5,7 @@ import fire
 import logging
 from .logger import LoggerProcess
 from .resizer import Resizer
+from .blurrer import BoundingBoxBlurrer
 from .writer import (
     WebDatasetSampleWriter,
     FilesSampleWriter,
@@ -23,6 +24,40 @@ import os
 logging.getLogger("exifread").setLevel(level=logging.CRITICAL)
 
 
+def arguments_validator(params):
+    """Validate the arguments"""
+    if params["compute_hash"] not in [None, "md5", "sha256", "sha512"]:
+        hash_type = params["compute_hash"]
+        raise ValueError(f"Unsupported hash to compute: {hash_type}")
+
+    if params["save_additional_columns"] is not None:
+        save_additional_columns_set = set(params["save_additional_columns"])
+
+        forbidden_columns = set(
+            [
+                "key",
+                "caption",
+                "url",
+                "width",
+                "height",
+                "original_width",
+                "original_height",
+                "status",
+                "error_message",
+                "exif",
+                "md5",
+                "sha256",
+                "sha512",
+            ]
+        )
+        intersection = save_additional_columns_set.intersection(forbidden_columns)
+        if intersection:
+            raise ValueError(
+                f"You cannot use in save_additional_columns the following columns: {intersection}."
+                + "img2dataset reserves these columns for its own use. Please remove them from save_additional_columns."
+            )
+
+
 def download(
     url_list: str,
     image_size: int = 256,
@@ -39,6 +74,7 @@ def download(
     input_format: str = "txt",
     url_col: str = "url",
     caption_col: Optional[str] = None,
+    bbox_col: Optional[str] = None,
     thread_count: int = 256,
     number_sample_per_shard: int = 10000,
     extract_exif: bool = True,
@@ -47,18 +83,27 @@ def download(
     enable_wandb: bool = False,
     wandb_project: str = "img2dataset",
     oom_shard_count: int = 5,
-    compute_md5: bool = True,
+    compute_hash: Optional[str] = "sha256",
     distributor: str = "multiprocessing",
     subjob_size: int = 1000,
     retries: int = 0,
     disable_all_reencoding: bool = False,
     min_image_size: int = 0,
+    max_image_area: float = float("inf"),
     max_aspect_ratio: float = float("inf"),
     incremental_mode: str = "incremental",
     max_shard_retry: int = 1,
+    user_agent_token: Optional[str] = None,
+    disallowed_header_directives: Optional[List[str]] = None,
 ):
     """Download is the main entry point of img2dataset, it uses multiple processes and download multiple files"""
+    if disallowed_header_directives is None:
+        disallowed_header_directives = ["noai", "noimageai", "noindex", "noimageindex"]
+    if len(disallowed_header_directives) == 0:
+        disallowed_header_directives = None
+
     config_parameters = dict(locals())
+    arguments_validator(config_parameters)
 
     def make_path_absolute(path):
         fs, p = fsspec.core.url_to_fs(path)
@@ -106,6 +151,12 @@ def download(
     logger_process.done_shards = done_shards
     logger_process.start()
 
+    if bbox_col is not None:
+        if save_additional_columns is None:
+            save_additional_columns = [bbox_col]
+        else:
+            save_additional_columns.append(bbox_col)
+
     reader = Reader(
         url_list,
         input_format,
@@ -130,13 +181,10 @@ def download(
     else:
         raise ValueError(f"Invalid output format {output_format}")
 
-    if encode_format not in ["jpg", "png", "webp"]:
-        raise ValueError(f"Invalid encode format {encode_format}")
-    if encode_format == "png":
-        if encode_quality < 0 or encode_quality > 9:
-            raise ValueError(
-                f"For png, encode quality represents compression which must be between 0 and 9, got {encode_quality}"
-            )
+    if bbox_col is not None:
+        blurrer = BoundingBoxBlurrer()
+    else:
+        blurrer = None
 
     resizer = Resizer(
         image_size=image_size,
@@ -149,7 +197,9 @@ def download(
         skip_reencode=skip_reencode,
         disable_all_reencoding=disable_all_reencoding,
         min_image_size=min_image_size,
+        max_image_area=max_image_area,
         max_aspect_ratio=max_aspect_ratio,
+        blurrer=blurrer,
     )
 
     downloader = Downloader(
@@ -163,9 +213,12 @@ def download(
         timeout=timeout,
         number_sample_per_shard=number_sample_per_shard,
         oom_shard_count=oom_shard_count,
-        compute_md5=compute_md5,
+        compute_hash=compute_hash,
         encode_format=encode_format,
         retries=retries,
+        user_agent_token=user_agent_token,
+        disallowed_header_directives=disallowed_header_directives,
+        blurring_bbox_col=bbox_col,
     )
 
     print("Starting the downloading of this file")
